# encoding: utf-8
"""
Render a template variable as Django template source in the current context
"""

from __future__ import absolute_import, division, print_function, unicode_literals

from django import template

register = template.Library()


class RenderInlineNode(template.Node):
    def __init__(self, nodelist):
        self.nodelist = nodelist

    def render(self, context):
        source = self.nodelist.render(context)
        t = template.Template(source)
        return t.render(context)


@register.tag
def render_inline(parser, token):
    """
    A template tag which renders its contents as a template using the current context, allowing you to process
    template code stored in something like gettext blocks, model content, ``django-flatblocks``, etc. as if
    it was actually in the current template.

    .. note: this could use things like ``|safe`` and is not safe to use on untrusted data

    Usage:

        my_template.html::

            {% render_inline %}
            <b>{{ VARIABLE_WHICH_CONTAINS_TEMPLATE_MARKUP }}</b>
            {% end_render_inline %}

        Context::

            {'VARIABLE_WHICH_CONTAINS_TEMPLATE_MARKUP': 'Hello {{ CURRENT_USER.name }}'}

        Output::

            '<b>Hello J. Random User</b>'
    """

    nodelist = parser.parse(('end_render_inline', ))

    parser.delete_first_token()

    return RenderInlineNode(nodelist)



# encoding: utf-8
# This module is courtesy of Adrian Macneil and licensed under the same terms as the Django codebase to
# which it was contributed:
# https://gist.github.com/amacneil/5af7cd0e934f5465b695
# https://code.djangoproject.com/ticket/17419

from __future__ import absolute_import, division, print_function, unicode_literals

from json import dumps as json_dumps

from django.core.serializers.json import DjangoJSONEncoder
from django.template import Library

register = Library()


@register.filter
def json(data):
    """
    Safely JSON-encode an object

    To protect against XSS attacks, HTML special characters (``<``, ``>``, ``&``) and unicode newlines are
    replaced by escaped unicode characters. Django does not escape these characters by default.

    Output of this method is not marked as HTML safe. If you use it inside an HTML attribute, it must be
    escaped like regular data::

        <div data-user="{{ data|json }}">

    If you use it inside a ``<script>`` tag, then the output does not need to be escaped, so you can mark it
    as safe::

        <script>
            var user = {{ data|json|safe }};
        </script>

    Escaped characters taken from Rails ``json_escape()`` helper:
    https://github.com/rails/rails/blob/v4.2.5/activesupport/lib/active_support/core_ext/string/output_safety.rb#L60-L113
    """

    unsafe_chars = {'&': '\\u0026',
                    '<': '\\u003c',
                    '>': '\\u003e',
                    '\u2028': '\\u2028',
                    '\u2029': '\\u2029'}

    json_str = json_dumps(data, cls=DjangoJSONEncoder)

    for (c, d) in unsafe_chars.items():
        json_str = json_str.replace(c, d)

    return json_str



# encoding: utf-8
"""
Query String manipulation filters
"""

from __future__ import absolute_import, division, print_function, unicode_literals

from django.http import QueryDict
from django.template import Library, Node, TemplateSyntaxError, Variable
from django.utils.html import escape
from django.utils.translation import ugettext as _

register = Library()


class QueryStringAlterer(Node):
    """
    Query String alteration template tag

    Receives a query string (either text or a QueryDict such as request.GET)
    and a list of changes to apply. The result will be returned as text query
    string, allowing use like this::

        <a href="?{% qs_alter request.GET type=object.type %}">{{ label }}</a>

    There are two available alterations:

        Assignment:

            name=var

        Deletion - removes the named parameter:

            delete:name

        Delete a parameter matching a value:

            delete_value:"name",value

        Delete a parameter matching a value from another variable:

            delete_value:field_name,value

    Examples:

    Query string provided as QueryDict::

        {% qs_alter request.GET foo=bar %}
        {% qs_alter request.GET foo=bar baaz=quux %}
        {% qs_alter request.GET foo=bar baaz=quux delete:corge %}

    Remove one facet from a list::

        {% qs_alter request.GET foo=bar baaz=quux delete_value:"facets",value %}

    Query string provided as string::

        {% qs_alter "foo=baaz" foo=bar %}

    Any query string may be stored in a variable in the local template context by making the last
    argument "as variable_name"::

        {% qs_alter request.GET foo=bar baaz=quux delete:corge as new_qs %}
    """

    def __init__(self, base_qs, as_variable, *args):
        self.base_qs = Variable(base_qs)
        self.args = args
        # Control whether we return the result or save it to the context:
        self.as_variable = as_variable

    def render(self, context):
        base_qs = self.base_qs.resolve(context)

        if isinstance(base_qs, QueryDict):
            qs = base_qs.copy()
        else:
            qs = QueryDict(base_qs, mutable=True)

        for arg in self.args:
            if arg.startswith("delete:"):
                v = arg[7:]
                if v in qs:
                    del qs[v]
            elif arg.startswith("delete_value:"):
                field, value = arg[13:].split(",", 2)
                value = Variable(value).resolve(context)

                if not (field[0] == '"' and field[-1] == '"'):
                    field = Variable(field).resolve(context)
                else:
                    field = field.strip('"\'')

                f_list = qs.getlist(field)
                if value in f_list:
                    f_list.remove(value)
                    qs.setlist(field, f_list)

            else:
                k, v = arg.split("=", 2)
                qs[k] = Variable(v).resolve(context)

        encoded_qs = escape(qs.urlencode())
        if self.as_variable:
            context[self.as_variable] = encoded_qs
            return ""
        else:
            return encoded_qs

    @classmethod
    def qs_alter_tag(cls, parser, token):
        try:
            bits = token.split_contents()
        except ValueError:
            raise TemplateSyntaxError(_('qs_alter requires at least two arguments: the initial query string'
                                        ' and at least one alteration'))

        if bits[-2] == 'as':
            as_variable = bits[-1]
            bits = bits[0:-2]
        else:
            as_variable = None

        return QueryStringAlterer(bits[1], as_variable, *bits[2:])

register.tag('qs_alter', QueryStringAlterer.qs_alter_tag)


class FacetAlterer(Node):
    """
    Search Facet-oriented querystring manipulation tag

    There are two tags (`add_facet` and `remove_facet`) which accept three
    arguments:

    :param original:
        initial querystring (most commonly a QueryDict like `request.GET` but
        may also be a simple dictionary)

    :param name:
        Either a variable containing the name of the field or a quoted literal
        value

    :param value:
        Either a variable containing the value or a quoted literal value

    Both add and remove safely handle multiple values by adding or removing the
    provided value while preserving whatever other values were present in the
    provided initial querystring.

    Examples:

    The most basic form, using literal values::

        <a href="?{% add_facet request.GET "item_type" "book" %}">Books</a>

    Using a variable key and value::

        <a href="?{% add_facet request.GET field_name field_value %}">{{ field_value }}</a>
    """

    def __init__(self, mode, original_querystring, key, value, *args):
        self.original_querystring = Variable(original_querystring)

        self.key = key
        self.value = value

        if mode not in ('add', 'remove'):
            raise TemplateSyntaxError('mode must be either add or remove')

        self.mode = mode

        self.output_variable_name = None

        if args:
            if len(args) == 2 and args[0] == 'as':
                self.output_variable_name = args[1]
            else:
                raise TemplateSyntaxError('No optional arguments other than `as output_variable` are allowed')

    def render(self, context):
        original_querystring = self.original_querystring.resolve(context)

        if isinstance(original_querystring, QueryDict):
            qs = original_querystring.copy()
        else:
            qs = QueryDict(original_querystring, mutable=True)

        qs.pop("page", None)

        key = self.key
        val = self.value

        if not (key[0] == key[-1] == '"'):
            key = Variable(key).resolve(context)
        else:
            key = key.strip('"')

        if not (val[0] == val[-1] == '"'):
            val = Variable(val).resolve(context)
        else:
            val = val.strip('"')

        field_values = qs.getlist(key)

        if self.mode == "add":
            field_values.append(val)
        else:
            if val in field_values:
                field_values.remove(val)

        qs.setlist(key, field_values)

        output = escape(qs.urlencode())

        if self.output_variable_name:
            context[self.output_variable_name] = output
        else:
            return output

    @classmethod
    def add_facet(cls, parser, token):
        try:
            bits = token.split_contents()
        except ValueError:
            raise TemplateSyntaxError('add_facet requires at least three '
                                      'arguments: the initial query string, '
                                      'the field and the new value')

        return FacetAlterer("add", bits[1], *bits[2:])

    @classmethod
    def remove_facet(cls, parser, token):
        try:
            bits = token.split_contents()
        except ValueError:
            raise TemplateSyntaxError('remove_facet requires at least three '
                                      'arguments: the initial query string, '
                                      'the field and the value to remove')

        return FacetAlterer("remove", bits[1], *bits[2:])

register.tag('add_facet', FacetAlterer.add_facet)
register.tag('remove_facet', FacetAlterer.remove_facet)



# encoding: utf-8
"""Utilities for manipulating the current template context"""
from __future__ import absolute_import, division, print_function, unicode_literals

from django import template

register = template.Library()


@register.filter
def get_key(dictlike, key):
    """Filter to return a dictionary value by name

    Returns ``None`` if the key does not exist so either check or use ``|default_if_none``

    Usage::

        {{ my_dict|get_key:"the key I want" }}
    """

    return dictlike.get(key, None)



# encoding: utf-8
from __future__ import absolute_import, division, print_function, unicode_literals

from itertools import chain


def validated_get_or_create(klass, **kwargs):
    """
    Similar to :meth:`~django.db.models.query.QuerySet.get_or_create` but uses
    the methodical get/save including a full_clean() call to avoid problems with
    models which have validation requirements which are not completely enforced
    by the underlying database.

    For example, with a django-model-translation we always want to go through
    the setattr route rather than inserting into the database so translated
    fields will be mapped according to the active language. This avoids normally
    impossible situations such as creating a record where `title` is defined but
    `title_en` is not.
    """

    defaults = kwargs.pop('defaults', {})

    try:
        obj = klass.objects.get(**kwargs)
        return obj, False
    except klass.DoesNotExist:
        obj = klass()

        for k, v in chain(kwargs.items(), defaults.items()):
            setattr(obj, k, v)

        obj.full_clean()
        obj.save()
        return obj, True



#!/usr/bin/env python
# encoding: utf-8
import os
import re
import sys

from setuptools import setup

readme = open('README.rst').read()

setup(
    name='django-bittersweet',
    version='1.0.0',
    description="""Reusable Django model validation utility""",
    long_description=readme,
    author='Chris Adams',
    author_email='chris@improbable.org',
    url='https://github.com/acdha/django-bittersweet',
    packages=[
        'bittersweet',
    ],
    include_package_data=True,
    install_requires=[
        'Django'
    ],
    test_suite='tests.run_tests.run_tests',
    license='CC0',
    zip_safe=False,
    keywords='bittersweet',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Intended Audience :: Developers',
        'License :: CC0 1.0 Universal (CC0 1.0) Public Domain Dedication',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
)
